package org.communiquons.dccaggregator.apis

import com.google.gson.Gson
import com.lunabeestudio.framework.crypto.BouncyCastleCryptoDataSource
import com.lunabeestudio.framework.crypto.ServerKeyAgreementHelper
import org.communiquons.dccaggregator.DCC_LIGHT_BASE_URL
import org.communiquons.dccaggregator.data.Certificate
import retrofit2.Response
import retrofit2.http.Body
import retrofit2.http.POST

internal class ApiAggregateRQ(
    val publicKey: String,
    val certificates: List<String>,
)

internal class ApiAggregateRS(
    val response: String,
)

internal class ApiAggregateCertificate(
    val certificate: String,
)


internal interface DccLightApi {
    @POST("/api/v1/aggregate")
    suspend fun aggregate(
        @Body apiAggregateRQ: ApiAggregateRQ,
    ): Response<ApiAggregateRS>
}

sealed class AggregateResult {
    class Error(val error: String, val throwable: Throwable?) : AggregateResult()
    class Success(val cert: Certificate) : AggregateResult()
}

class DccLightAPIClient {
    private val api by lazy {
        getRetrofitService(DCC_LIGHT_BASE_URL, DccLightApi::class.java)
    }

    private val serverKeyAgreementHelper by lazy {
        ServerKeyAgreementHelper(BouncyCastleCryptoDataSource())
    }

    private val gson = Gson()

    suspend fun aggregateCertificates(
        serverPubKey: String,
        certs: List<Certificate>
    ): AggregateResult {
        // Encrypt request
        val serverKeyAgreementData = try {
            serverKeyAgreementHelper.encryptRequestData(serverPubKey, certs.map { c -> c.encoded })
        } catch (e: Exception) {
            return AggregateResult.Error(e.message ?: "Failed to cipher request", e)
        }

        val apiAggregateRQ = ApiAggregateRQ(
            serverKeyAgreementData.encodedLocalPublicKey,
            serverKeyAgreementData.encryptedData,
        )

        @Suppress("BlockingMethodInNonBlockingContext")
        return try {
            val response = api.aggregate(apiAggregateRQ)
            if (response.isSuccessful) {
                response.body()?.let {
                    val multipassJson = serverKeyAgreementHelper.decryptResponse(it.response)
                    val multipass = gson.fromJson(
                        multipassJson,
                        ApiAggregateCertificate::class.java
                    ).certificate
                    AggregateResult.Success(Certificate(multipass))
                } ?: AggregateResult.Error(
                    "Response successful (${response.code()}) but body is empty",
                    null
                )

            } else {
                val error = response.errorBody()?.string()
                if (error != null) {
                    AggregateResult.Error("Aggregate request failed! $error", null)
                } else {
                    AggregateResult.Error("Request failed with status ${response.code()}", null)
                }
            }
        } catch (e: Exception) {
            return AggregateResult.Error("Request failed", e)
        }
    }
}